@@ -5,6 +5,14 @@
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE QuasiQuotes #-}
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE ViewPatterns #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
+module Main
+    ( main
+    -- avoid warnings
+    , resourcesRoutedApp
+    , Widget
+    ) where
 
 import Test.HUnit hiding (Test)
 import Test.Hspec
@@ -21,16 +29,25 @@ import Control.Applicative
 import Network.Wai (pathInfo, requestHeaders)
 import Data.Maybe (fromMaybe)
 import Data.Either (isLeft, isRight)
-import Control.Exception.Lifted(try, SomeException)
 
 import Data.ByteString.Lazy.Char8 ()
 import qualified Data.Map as Map
 import qualified Text.HTML.DOM as HD
 import Network.HTTP.Types.Status (status301, status303, unsupportedMediaType415)
 
+parseQuery_ :: Text -> [[SelectorGroup]]
 parseQuery_ = either error id . parseQuery
+
+findBySelector_ :: HtmlLBS -> Query -> [String]
 findBySelector_ x = either error id . findBySelector x
-parseHtml_ = HD.parseLBS
+
+data RoutedApp = RoutedApp
+
+mkYesod "RoutedApp" [parseRoutes|
+/                HomeR      GET POST
+/resources       ResourcesR POST
+/resources/#Text ResourceR  GET
+|]
 
 main :: IO ()
 main = hspec $ do
@@ -77,7 +94,7 @@ main = hspec $ do
                             [NodeContent "Hello World"]
                         ]
                     ]
-             in parseHtml_ html @?= doc
+             in HD.parseLBS html @?= doc
         it "HTML" $
             let html = "<html><head><title>foo</title></head><body><br><p>Hello World</p></body></html>"
                 doc = Document (Prologue [] Nothing []) root []
@@ -92,7 +109,7 @@ main = hspec $ do
                             [NodeContent "Hello World"]
                         ]
                     ]
-             in parseHtml_ html @?= doc
+             in HD.parseLBS html @?= doc
     describe "basic usage" $ yesodSpec app $ do
         ydescribe "tests1" $ do
             yit "tests1a" $ do
@@ -209,7 +226,7 @@ main = hspec $ do
             statusIs 200
             printBody
             bodyContains "Foo"
-    describe "CSRF with cookies/headers" $ yesodSpec CsrfApp $ do
+    describe "CSRF with cookies/headers" $ yesodSpec RoutedApp $ do
         yit "Should receive a CSRF cookie and add its value to the headers" $ do
             get ("/" :: Text)
             statusIs 200
@@ -251,6 +268,30 @@ main = hspec $ do
             r <- followRedirect
             liftIO $ assertBool "expected a Left when not a redirect" $ isLeft r
 
+    describe "route parsing in tests" $ yesodSpec RoutedApp $ do
+        yit "parses location header into a route" $ do
+            -- get CSRF token
+            get HomeR
+            statusIs 200
+
+            request $ do
+                setMethod "POST"
+                setUrl $ ResourcesR
+                addPostParam "foo" "bar"
+                addTokenFromCookie
+            statusIs 201
+
+            loc <- getLocation
+            liftIO $ assertBool "expected location to be available" $ isRight loc
+            let (Right (ResourceR t)) = loc
+            liftIO $ assertBool "expected location header to contain post param" $ t == "bar"
+
+        yit "returns a Left when no redirect was returned" $ do
+            get HomeR
+            statusIs 200
+            loc <- getLocation
+            liftIO $ assertBool "expected a Left when not a redirect" $ isLeft loc
+
 instance RenderMessage LiteApp FormMessage where
     renderMessage _ _ = defaultFormMessage
 
@@ -277,7 +318,7 @@ app = liteApp $ do
         ((mfoo, widget), _) <- runFormPost
                         $ renderDivs
                         $ (,)
-                      <$> areq textField "Some Label" Nothing
+                      Control.Applicative.<$> areq textField "Some Label" Nothing
                       <*> areq fileField "Some File" Nothing
         case mfoo of
             FormSuccess (foo, _) -> return $ toHtml foo
@@ -304,16 +345,10 @@ cookieApp = liteApp $ do
     onStatic "cookie" $ do
         onStatic "foo" $ dispatchTo $ do
             setMessage "Foo"
-            redirect ("/cookie/home" :: Text)
+            () <- redirect ("/cookie/home" :: Text)
             return ()
 
-data CsrfApp = CsrfApp
-
-mkYesod "CsrfApp" [parseRoutes|
-/ HomeR GET POST
-|]
-
-instance Yesod CsrfApp where
+instance Yesod RoutedApp where
     yesodMiddleware = defaultYesodMiddleware . defaultCsrfMiddleware
 
 getHomeR :: Handler Html
@@ -329,3 +364,15 @@ postHomeR = defaultLayout
         <p>
             Welcome to my test application.
     |]
+
+postResourcesR :: Handler ()
+postResourcesR = do
+  ([("foo", t)], _) <- runRequestBody
+  sendResponseCreated $ ResourceR t
+
+getResourceR :: Text -> Handler Html
+getResourceR i = defaultLayout
+    [whamlet|
+        <p>
+            Read item #{i}.
+    |]
